{$O+A+W-U-I+B-X+T-P+H+J+R-Q-D-L-}

//******************************************************
//******************************************************
// Project: Delphi Initialization Database
// Author: JBD
// E-mail: binel@alf.tel.hr
// Comment: Some additional "INI" sections
//******************************************************
//******************************************************

unit INISecEx;

interface

uses Classes,Windows,Graphics,Forms,Dialogs,ExtCtrls,SysUtils,
			INISec,INIEnt,INIEntEx,INIMis,TypInfo;

type
	TINISectionFont=class(TINICustomSection)
  private
  	FDefaultFont:TFont;
    FPreviewHeight:Integer;
    FPreviewVisible,FFontButtonVisible:Boolean;

  	procedure OpenFontDialog(Sender:TObject);

  	procedure PreviewPaint(Sender:TObject);
  	procedure RefreshPreview(Sender:TINIField);

		procedure SetDefaultValues;

    procedure SetDefaultFont(Font:TFont);
    procedure SetPreviewHeight(H:Integer);
    procedure SetFontButtonVisible(pVisible:Boolean);
    procedure SetPreviewVisible(pVisible:Boolean);

  protected
  	procedure Loaded; override;

		procedure Reset; override;

  public
  	constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;

		procedure GetFont(Font:TFont);
    procedure SetFont(Font:TFont);

  published
  	property DefaultFont:TFont read FDefaultFont write SetDefaultFont;
    property PreviewHeight:Integer read FPreviewHeight write SetPreviewHeight default 64;
    property PreviewVisible:Boolean read FPreviewVisible write SetPreviewVisible default True;
    property FontButtonVisible:Boolean read FFontButtonVisible write SetFontButtonVisible default False;
    property ImageIndex;
  end;

	TINIMultiSectionFormSaver=class(TINICustomMultiSection)
  public
  	constructor Create(AOwner:TComponent); override;

  published
		property OnButtonClick;

		property CompressKey;
    property Static;
		property DropDownCount;
    property Buttons;
    property ImageIndex;
  end;

	TINISectionComponent=class(TINISection)
  public
		procedure SetObjectProperties(C:TObject);
		procedure GetObjectProperties(C:TObject);

  published
    property ImageIndex;
	end;

procedure Register;

implementation

uses INISaver;

constructor TINISectionFont.Create(AOwner:TComponent);
begin
	inherited Create(AOwner);

  FPreviewHeight:=64;
  FPreviewVisible:=True;
  FFontButtonVisible:=False;

  RenameGroupByIndex(0,'Font');

  GroupType:=gtCascade;

  with TINIEntryFontName.Create(Self) do begin
  	Table:=Self;
  	FieldName:='FontName';
    DisplayLabel:='Name';
    DisplayWidth:=16;
    GroupName:='Font';
    OnChange:=RefreshPreview;
  end;

  with TINIEntryColor.Create(Self) do begin
  	Table:=Self;
  	FieldName:='Color';
    GroupName:='Font';
    DisplayWidth:=16;
    DefaultValue:=clBlack;
    OnChange:=RefreshPreview;
  end;

  with TINIEntryInteger.Create(Self) do begin
  	Table:=Self;
  	FieldName:='Size';
    GroupName:='Font';
    DisplayWidth:=4;
    IMinValue:=4;
    IMaxValue:=78;
    DefaultValue:=8;
    OnChange:=RefreshPreview;
  end;

  with TINIEntryManyOfStrings.Create(Self) do begin
  	Table:=Self;
  	FieldName:='Style';
    GroupName:='Font';
    DisplayWidth:=0;
    CaptionStyle:=csTopLeft;
    BorderStyle:=bsSingle;
    AddItem('0','Bold');
    AddItem('1','Italic');
    AddItem('2','Underline');
    AddItem('3','StrikeOut');
    DefaultValue:='0000';
    OnChange:=RefreshPreview;
  end;

  with TINIEntryPaintBox.Create(Self) do begin
  	Table:=Self;
  	FieldName:='Preview';
    GroupName:='Font';
    Width:=0;
    Height:=FPreviewHeight;
    CaptionStyle:=csTopLeft;
    OnPaint:=PreviewPaint;
  end;

  with TINIEntryString.Create(Self) do begin
  	Table:=Self;
    FieldName:='SampleText';
    DisplayLabel:='Sample text';
    DisplayWidth:=0;
    CaptionStyle:=csTopLeft;
    DefaultValue:='Binel d.o.o.';
    OnChange:=RefreshPreview;
  end;

  with TINIEntryButton.Create(Self) do begin
  	Table:=Self;
    FieldName:='FontButton';
    DisplayLabel:='Font dialog';
    Width:=72;
    Height:=24;
    Visible:=False;
    ShowCaption:=True;
    CaptionStyle:=csWithout;
    OnClick:=OpenFontDialog;
  end;

  FDefaultFont:=TFont.Create;

  if AOwner is TForm then FDefaultFont.Assign((AOwner as TForm).Font);
end;

destructor TINISectionFont.Destroy;
begin
	FDefaultFont.Free;

  inherited Destroy;
end;

procedure TINISectionFont.Reset;
begin
	GetFont(FDefaultFont);	
end;

procedure TINISectionFont.SetFont(Font:TFont);
begin
	Font.Name:=FieldByName('FontName').AsString;
	Font.Size:=FieldByName('Size').AsInteger;
	Font.Color:=(FieldByName('Color') as TINIEntryColor).AsColor;

  Font.Style:=[];

  with FieldByName('Style') as TINIEntryManyOfStrings do begin
  	if Checked[0] then Font.Style:=Font.Style+[fsBold];
  	if Checked[1] then Font.Style:=Font.Style+[fsItalic];
  	if Checked[2] then Font.Style:=Font.Style+[fsUnderline];
  	if Checked[3] then Font.Style:=Font.Style+[fsStrikeOut];
  end;
end;

procedure TINISectionFont.GetFont(Font:TFont);
begin
	FieldByName('FontName').AsString:=Font.Name;
	FieldByName('Size').AsInteger:=Font.Size;
	(FieldByName('Color') as TINIEntryColor).AsColor:=Font.Color;

  with FieldByName('Style') as TINIEntryManyOfStrings do begin
  	Checked[0]:=fsBold in Font.Style;
  	Checked[1]:=fsItalic in Font.Style;
  	Checked[2]:=fsUnderline in Font.Style;
  	Checked[3]:=fsStrikeOut in Font.Style;
  end;
end;

procedure TINISectionFont.PreviewPaint(Sender:TObject);
begin
	with FieldByName('Preview') as TINIEntryPaintBox do begin
  	if PaintBox<>nil then with PaintBox do begin
    	SetFont(Canvas.Font);

      PatBlt(Canvas.Handle,1,1,Width-2,Height-2,WHITENESS);

    	Canvas.Brush.Color:=clWhite;
      Canvas.TextOut(5,5,FieldByName('SampleText').AsString);
    	Canvas.Brush.Color:=clBlack;
      Canvas.FrameRect(Rect(0,0,Width,Height));
    end;
  end;
end;

procedure TINISectionFont.RefreshPreview(Sender:TINIField);
begin
	PreviewPaint(nil);
end;

procedure TINISectionFont.SetDefaultFont(Font:TFont);
begin
	FDefaultFont.Assign(Font);

  SetDefaultValues;
end;

procedure TINISectionFont.Loaded;
begin
	inherited Loaded;

  SetDefaultValues;
end;

procedure TINISectionFont.SetDefaultValues;
begin
  TINIEntryFontName(FieldByName('FontName')).DefaultValue:=FDefaultFont.Name;
  TINIEntryInteger(FieldByName('Size')).DefaultValue:=FDefaultFont.Size;
  TINIEntryColor(FieldByName('Color')).DefaultValue:=FDefaultFont.Color;

  with TINIEntryManyOfStrings(FieldByName('Style')) do begin
  	DefaultValue:='';

    DefaultValue:=DefaultValue+strIf(fsBold in FDefaultFont.Style,'1','0');
    DefaultValue:=DefaultValue+strIf(fsItalic in FDefaultFont.Style,'1','0');
    DefaultValue:=DefaultValue+strIf(fsUnderline in FDefaultFont.Style,'1','0');
    DefaultValue:=DefaultValue+strIf(fsStrikeOut in FDefaultFont.Style,'1','0');
  end;

  Reset;
end;

procedure TINISectionFont.SetPreviewHeight(H:Integer);
begin
	if H>=0 then begin
		FPreviewHeight:=H;

    TINIEntryPaintBox(FieldByName('Preview')).Height:=H;
  end;
end;

procedure TINISectionFont.SetFontButtonVisible(pVisible:Boolean);
begin
	FFontButtonVisible:=pVisible;

  FieldByName('FontButton').Visible:=pVisible;
end;

procedure TINISectionFont.SetPreviewVisible(pVisible:Boolean);
begin
	FPreviewVisible:=pVisible;

  FieldByName('Preview').Visible:=pVisible;
  FieldByName('SampleText').Visible:=pVisible;
end;

procedure TINISectionFont.OpenFontDialog(Sender:TObject);
var FD:TFontDialog;
begin
	FD:=TFontDialog.Create(Self);

  try
  	SetFont(FD.Font);
    
  	if FD.Execute then begin
      TINIEntryFontName(FieldByName('FontName')).AsString:=FD.Font.Name;
      TINIEntryInteger(FieldByName('Size')).AsInteger:=FD.Font.Size;
      TINIEntryColor(FieldByName('Color')).AsColor:=FD.Font.Color;

      with TINIEntryManyOfStrings(FieldByName('Style')) do begin
        Checked[0]:=fsBold in FD.Font.Style;
        Checked[1]:=fsItalic in FD.Font.Style;
        Checked[2]:=fsUnderline in FD.Font.Style;
        Checked[3]:=fsStrikeOut in FD.Font.Style;
      end;

      RefreshControls;
    end;
     
  	finally
	  	FD.Free;
  end;
end;

constructor TINIMultiSectionFormSaver.Create(AOwner:TComponent);
begin
	inherited Create(AOwner);

  RenameGroupByIndex(0,'Form');

  with TINIEntryInteger.Create(Self) do begin
  	Table:=Self;
  	FieldName:='X';
    DisplayLabel:='X';
    DisplayWidth:=6;
    DefaultValue:=0;
    IMinValue:=0;
    IMaxValue:=2048;
  end;

  with TINIEntryInteger.Create(Self) do begin
  	Table:=Self;
  	FieldName:='Y';
    DisplayLabel:='Y';
    DisplayWidth:=6;
    DefaultValue:=0;
    IMinValue:=0;
    IMaxValue:=2048;
  end;

  with TINIEntryInteger.Create(Self) do begin
  	Table:=Self;
  	FieldName:='Width';
    DisplayLabel:='Width';
    DisplayWidth:=6;
    DefaultValue:=360;
    IMinValue:=0;
    IMaxValue:=2048;
  end;

  with TINIEntryInteger.Create(Self) do begin
  	Table:=Self;
  	FieldName:='Height';
    DisplayLabel:='Height';
    DisplayWidth:=6;
    DefaultValue:=360;
    IMinValue:=0;
    IMaxValue:=2048;
  end;

  with TINIEntryOneOfStrings.Create(Self) do begin
  	Table:=Self;
  	FieldName:='State';
    AddItem('0','Normal');
    AddItem('1','Minimized');
    AddItem('2','Maximized');
    DisplayWidth:=12;
    DefaultValue:='0';
  end;
end;

procedure TINISectionComponent.SetObjectProperties(C:TObject);
var PClass:TClass;

	function StringToBinary(const Value:string):LongInt;
  var i,vLen:Integer;
  begin
  	Result:=0;

    vLen:=Length(Value);

    for i:=vLen downto 1 do begin
    	Result:=Result shl 1;

    	if Value[i]<>'0' then Result:=Result+1;
    end;
  end;

  procedure SetProperties(Section:TINICustomSection;TI:PTypeInfo;C:TObject);
  var PL:TPropList;
  		TK:TTypeKind;
      nEntry:TINIField;
      nSection:TINICustomSection;
      i,pc:Integer;
	    PTI:PTypeInfo;
	    PI:PPropInfo;
      lC:TObject;

  begin
    pc:=GetPropList(TI,tkProperties,@PL);

    for i:=0 to pc-1 do begin
     	PI:=PL[i];

{$ifdef VER90}
			PTI:=PI.PropType;
{$endif}
{$ifndef VER90}
			PTI:=PI.PropType^;
{$endif}

      nEntry:=Section.FindField(PI.Name);

    	TK:=PTI.Kind;

      if nEntry<>nil then begin
      	try
          if TK in [tkEnumeration,tkInteger] then
            SetOrdProp(C,PI,nEntry.AsInteger)
          else if TK=tkSet then
            SetOrdProp(C,PI,StringToBinary(nEntry.AsString))
          else if TK in [tkWChar,tkLString{$ifdef VER90},tkLWString{$endif}] then
            SetStrProp(C,PI,nEntry.AsString)
          else if TK=tkFloat then
            SetFloatProp(C,PI,nEntry.AsFloat)
          else if (TK=tkClass) and (nEntry is TINIEntrySaveList) then begin
            if (PTI=TStrings.ClassInfo) or (PTI=TStringList.ClassInfo) then begin
            	TStrings(GetOrdProp(C,PI)).Assign((nEntry as TINIEntrySaveList).List);
            end;
          end;

          except
            on EAccessViolation do begin
            end;
        end;
      end else if TK=tkClass then begin
      	lC:=nil;
        
        try
          lC:=TObject(GetOrdProp(C,PI));

          except
            on Exception do lC:=nil;
        end;

        nSection:=Section.FindSection(PI.Name);

        if (nSection<>nil) and (lC<>nil) and (lC.ClassType<>nil) then begin
          SetProperties(nSection,lC.ClassType.ClassInfo,lC);
        end;
      end;
    end;
  end;

begin
  PClass:=C.ClassType;

	if PClass.ClassInfo<>nil then SetProperties(Self,PClass.ClassInfo,C);
end;

procedure TINISectionComponent.GetObjectProperties(C:TObject);
var PClass:TClass;

	function BinaryToString(Value:LongInt):string;
  begin
  	Result:='';

    while Value>0 do begin
    	Result:=Result+Chr(48+(Value mod 2));

      Value:=Value div 2;
    end;
  end;
  
	function StringToBinary(const Value:string):LongInt;
  var i,vLen:Integer;
  begin
  	Result:=0;

    vLen:=Length(Value);

    for i:=vLen downto 1 do begin
    	Result:=Result shl 1;

    	if Value[i]<>'0' then Result:=Result+1;
    end;
  end;

  procedure GetProperties(Section:TINICustomSection;TI:PTypeInfo;C:TObject);
  var PL:TPropList;
  		TK:TTypeKind;
      nEntry:TINIField;
      nSection:TINICustomSection;
      i,pc:Integer;
	    PTI:PTypeInfo;
	    PI:PPropInfo;
      lC:TObject;

  begin
    pc:=GetPropList(TI,tkProperties,@PL);

    for i:=0 to pc-1 do begin
     	PI:=PL[i];

{$ifdef VER90}
			PTI:=PI.PropType;
{$endif}
{$ifndef VER90}
			PTI:=PI.PropType^;
{$endif}

      nEntry:=Section.FindField(PI.Name);

    	TK:=PTI.Kind;

      if nEntry<>nil then begin
      	try
          if TK in [tkEnumeration,tkInteger] then
            nEntry.AsInteger:=GetOrdProp(C,PI)
          else if TK=tkSet then
            nEntry.AsString:=BinaryToString(GetOrdProp(C,PI))
          else if TK in [tkWChar,tkLString{$ifdef VER90},tkLWString{$endif}] then
            nEntry.AsString:=GetStrProp(C,PI)
          else if TK=tkFloat then
            nEntry.AsFloat:=GetFloatProp(C,PI)
          else if (TK=tkClass) and (nEntry is TINIEntrySaveList) then begin
            if (PTI=TStrings.ClassInfo) or (PTI=TStringList.ClassInfo) then begin
            	(nEntry as TINIEntrySaveList).List.Assign(TStrings(GetOrdProp(C,PI)));
            end;
          end;

          except
            on EAccessViolation do begin
            end;
        end;
      end else if TK=tkClass then begin
      	lC:=nil;

        try
          lC:=TObject(GetOrdProp(C,PI));

          except
            on Exception do lC:=nil;
        end;

        nSection:=Section.FindSection(PI.Name);

        if (nSection<>nil) and (lC<>nil) and (lC.ClassType<>nil) then begin
          GetProperties(nSection,lC.ClassType.ClassInfo,lC);
        end;
      end;
    end;
  end;

begin
  PClass:=C.ClassType;

	if PClass.ClassInfo<>nil then GetProperties(Self,PClass.ClassInfo,C);
end;

procedure Register;
begin
	RegisterINISection('Object properties',TINISectionComponent);
	RegisterINISection('Font',TINISectionFont);
	RegisterINISection('Form saver',TINIMultiSectionFormSaver);

  INISaver.Register;
end;

end.
