{$O+A+W-U-I+B-X+T-P+H+J+R-Q-D-L-}

//******************************************************
//******************************************************
// Project: Delphi Initialization Database
// Author: JBD
// E-mail: binel@alf.tel.hr
// Comment: Form for testing Initialization Database
//******************************************************
//******************************************************

unit INITest;

interface

uses
	SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
	Forms, Dialogs, StdCtrls, Buttons, INIUtil, ExtCtrls, 
	INIEnt,FSaver,INISec,INIDsgn;

type
	TINIDatabaseMode=(dmTest,dmEdit);

  TfrmTestINIDatabase=class(TForm)
    pnlBottom: TPanel;
    pnlButtons: TAutoAlignPanel;
    pnlMain: TPanel;
    FormSaver: TFormSaver;
    BBSave: TBitBtn;
    BBClose: TBitBtn;
    BBHelp: TBitBtn;
    procedure BBSaveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure BBHelpClick(Sender: TObject);

  private
  	FSaveCaption,FEditGroup:string;
  	FINIDatabase:TINIDatabase;
    FINISection,FEditINISection:TINICustomSection;

		procedure PrepareDatabase;

  protected
		procedure WMGetMinMaxInfo(var Msg:TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
    
  public
  	function GoTest(INIDatabase:TINIDatabase;Section,EditSection:TINICustomSection;const EditGroup:string;DM:TINIDatabaseMode;Resizable:Boolean):Integer;
  end;

var frmTestINIDatabase:TfrmTestINIDatabase;

implementation

{$R *.DFM}

uses INIMis;

function TfrmTestINIDatabase.GoTest(INIDatabase:TINIDatabase;Section,EditSection:TINICustomSection;const EditGroup:string;DM:TINIDatabaseMode;Resizable:Boolean):Integer;
begin
	FINIDatabase:=INIDatabase;
  FINISection:=Section;
  FEditINISection:=EditSection;
  FEditGroup:=EditGroup;

  if Resizable then
  	BorderStyle:=bsSizeable
  else
  	BorderStyle:=bsSingle;

  if FINIDatabase.EditOnlyMode then
  	Caption:=Format('%s [Edit Only]',[FSaveCaption])
  else
  	Caption:=FSaveCaption;

  try
  	Result:=ShowModal;

    finally
		  FINIDatabase.EditOnlyMode:=False;
  end;
end;

procedure TfrmTestINIDatabase.PrepareDatabase;
var savePW:TWinControl;
begin
  while pnlMain.ControlCount>0 do pnlMain.Controls[0].Free;

  savePW:=FINIDatabase.Parent;

  try
    FINIDatabase.EditOnlyMode:=(FINISection<>nil) and (FINISection.Son);

    FINIDatabase.Parent:=pnlMain;

    if FINISection<>nil then begin
      FINIDatabase.CreatePage(Self,FINISection,pnlMain);

      FINISection.EnableControls;

      BBSave.Enabled:=not FINIDatabase.EditOnlyMode;

     	FINISection.Init;
     	FINISection.Reset;

      if not FINIDatabase.EditOnlyMode then begin
        FINISection.ReadSection;
      end else begin
        FINISection.ClearControls;
      end;

      FINISection.RefreshControls;
    end else begin
      FINIDatabase.Init;
      FINIDatabase.Reset;
      FINIDatabase.Read;
      FINIDatabase.CreatePages;
      FINIDatabase.Refresh;
    end;

    if FEditINISection<>nil then FEditINISection.OpenGroupForEdit(FEditGroup); 

    finally
      FINIDatabase.Parent:=savePW;
  end;
end;

procedure TfrmTestINIDatabase.BBSaveClick(Sender: TObject);
begin
	if FINISection<>nil then
  	FINISection.WriteSection
  else
  	FINIDatabase.Write;
end;

procedure TfrmTestINIDatabase.FormCreate(Sender: TObject);
begin
	FSaveCaption:=Caption;
end;

procedure TfrmTestINIDatabase.FormShow(Sender: TObject);
begin
	PrepareDatabase;
end;

procedure TfrmTestINIDatabase.FormResize(Sender: TObject);
begin
	pnlButtons.Width:=Min(250,Self.ClientWidth);
end;

procedure TfrmTestINIDatabase.WMGetMinMaxInfo(var Msg:TWMGetMinMaxInfo);
begin
	inherited;

  with Msg.MinMaxInfo^.ptMinTrackSize do begin
  	x:=200;
    y:=140;
  end;
end;

procedure TfrmTestINIDatabase.BBHelpClick(Sender: TObject);
begin
	Application.HelpJump('TestingDatabase');
end;

end.
